﻿//Copyright (C) Microsoft Corporation.  All rights reserved.

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using IdeaBlade.EntityModel;

namespace SampleSupport {

  
  internal partial class SampleForm : Form {
    private SampleHarness currentHarness;
    private Sample currentSample;

    public static TreeView OutputTreeView { get; set; }

    #region Constructor (Some changes)
    public SampleForm(string title, List<SampleHarness> harnesses) {

      InitializeComponent();

      qsCacheOnlyMenuItem.Tag = QueryStrategy.CacheOnly;
      qsDataSourceOnlyMenuItem.Tag = QueryStrategy.DataSourceOnly;
      qsDataSourceThenCacheMenuItem.Tag = QueryStrategy.DataSourceThenCache;
      qsNormal.Tag = QueryStrategy.Normal;

      QueryStrategy = QueryStrategy.DataSourceOnly;

      // Added following two lines
      _harnesses = harnesses;
      var managers = _harnesses.OfType<DevForceSampleHarness>().Select(x => x.EntityManager).Distinct();
      if (managers.Count() != 1) {
        throw new Exception("Exactly one EntityManager instance is expected for this application");
      }
      _entityManager = managers.First();
      InitImageList();
      ConfigureStatusStrip();
      UpdateMenus();

      this.Text = title;

      TreeNode rootNode = new TreeNode(title);
      rootNode.Tag = null;
      rootNode.ImageKey = "Help";
      rootNode.SelectedImageKey = "Help";
      samplesTreeView.Nodes.Add(rootNode);
      rootNode.Expand();

      foreach (SampleHarness harness in harnesses) {
        TreeNode harnessNode = new TreeNode(harness.Title);
        harnessNode.Tag = null;
        harnessNode.ImageKey = "BookStack";
        harnessNode.SelectedImageKey = "BookStack";
        rootNode.Nodes.Add(harnessNode);

        string category = "";
        TreeNode categoryNode = null;
        foreach (Sample sample in harness) {
          if (sample.Category != category) {
            category = sample.Category;

            categoryNode = new TreeNode(category);
            categoryNode.Tag = null;
            categoryNode.ImageKey = "BookClosed";
            categoryNode.SelectedImageKey = "BookClosed";
            harnessNode.Nodes.Add(categoryNode);
          }

          TreeNode node = new TreeNode(sample.ToString());
          node.Tag = sample;
          node.ImageKey = "Item";
          node.SelectedImageKey = "Item";
          categoryNode.Nodes.Add(node);
        }
      }
    }

    #endregion

    public QueryStrategy QueryStrategy {
      get { return _queryStrategy; }
      set {
        _queryStrategy = value;
        UpdateQsMenu(_queryStrategy);
      }
    }

    private QueryStrategy _queryStrategy;

    private void samplesTreeView_AfterSelect(object sender, TreeViewEventArgs e) {
      TreeNode currentNode = samplesTreeView.SelectedNode;
      currentSample = (Sample)currentNode.Tag;
      if (currentSample != null) {
        currentHarness = currentSample.Harness;
        runButton.Enabled = true;
        descriptionTextBox.Text = currentSample.Description;
        codeRichTextBox.Clear();
        codeRichTextBox.Text = currentSample.Code;
        outputTextBox.Clear();

        treeViewOutput.Nodes.Clear();
        colorizeCode(codeRichTextBox);
      }
      else {
        currentHarness = null;
        runButton.Enabled = false;
        descriptionTextBox.Text = "Select a query from the tree to the left.";
        codeRichTextBox.Clear();
      }
    }

    private void samplesTreeView_AfterExpand(object sender, TreeViewEventArgs e) {
      switch (e.Node.Level) {
        case 1:
        case 2:
          e.Node.ImageKey = "BookOpen";
          e.Node.SelectedImageKey = "BookOpen";
          break;
      }
    }

    private void samplesTreeView_BeforeCollapse(object sender, TreeViewCancelEventArgs e) {
      switch (e.Node.Level) {
        case 0:
          e.Cancel = true;
          break;
      }
    }

    private void samplesTreeView_AfterCollapse(object sender, TreeViewEventArgs e) {
      switch (e.Node.Level) {
        case 1:
          e.Node.ImageKey = "BookStack";
          e.Node.SelectedImageKey = "BookStack";
          break;

        case 2:
          e.Node.ImageKey = "BookClosed";
          e.Node.SelectedImageKey = "BookClosed";
          break;
      }
    }

    private void samplesTreeView_DoubleClick(object sender, EventArgs e) {
      if (this.currentSample != null) {
        RunCurrentSample();
      }
    }

    private static void colorizeCode(RichTextBox rtb) {
      string[] keywords = {"as", "do", "if", "in", "is", "for", "int", "new", "out", "ref", "try", "base", 
                                "bool", "byte", "case", "char", "else", "enum", "goto", "lock", "long", "null", 
                                "this", "true", "uint", "void", "break", "catch", "class", "const", "event", "false", 
                                "fixed", "float", "sbyte", "short", "throw", "ulong", "using", "where", "while", 
                                "yield", "double", "extern", "object", "params", "public", "return", "sealed", 
                                "sizeof", "static", "string", "struct", "switch", "typeof", "unsafe", "ushort", 
                                "checked", "decimal", "default", "finally", "foreach", "partial", "private", 
                                "virtual", "abstract", "continue", "delegate", "explicit", "implicit", "internal", 
                                "operator", "override", "readonly", "volatile",  
                                "interface", "namespace", "protected", "unchecked",
                                "stackalloc", 
                                "from", "in", "where", "select", "join", "equals", "let", "on", "group", "by", 
                                "into", "orderby", "ascending", "descending", "var"};
      string text = rtb.Text;

      rtb.SelectAll();
      rtb.SelectionColor = rtb.ForeColor;

      foreach (String keyword in keywords) {
        int keywordPos = rtb.Find(keyword, RichTextBoxFinds.MatchCase | RichTextBoxFinds.WholeWord);
        while (keywordPos != -1) {
          int commentPos = text.LastIndexOf("//", keywordPos, StringComparison.OrdinalIgnoreCase);
          int newLinePos = text.LastIndexOf("\n", keywordPos, StringComparison.OrdinalIgnoreCase);
          int quoteCount = 0;
          int quotePos = text.IndexOf("\"", newLinePos + 1, keywordPos - newLinePos, StringComparison.OrdinalIgnoreCase);
          while (quotePos != -1) {
            quoteCount++;
            quotePos = text.IndexOf("\"", quotePos + 1, keywordPos - (quotePos + 1), StringComparison.OrdinalIgnoreCase);
          }

          if (newLinePos >= commentPos && quoteCount % 2 == 0)
            rtb.SelectionColor = Color.Blue;

          keywordPos = rtb.Find(keyword, keywordPos + rtb.SelectionLength, RichTextBoxFinds.MatchCase | RichTextBoxFinds.WholeWord);
        }
      }

      rtb.Select(0, 0);
    }

    private void treeViewOutput_BeforeExpand(object sender, TreeViewCancelEventArgs e) {
      if (_inBeforeExpand) {
        e.Cancel = true;
        return;
      }
      try {
        _inBeforeExpand = true;
        TreeViewGenerator.Instance.ExpandTreeNode(e.Node);
      } finally {
        _inBeforeExpand = false;
      }

    }
    private bool _inBeforeExpand = false;

    private void runButton_Click(object sender, EventArgs e) {
      RunCurrentSample();
    }

    private void RunCurrentSample() {

      treeViewOutput.Nodes.Clear();
      Cursor hold = this.Cursor;
      try {
        this.Cursor = Cursors.WaitCursor;
        OutputTreeView = treeViewOutput;
        currentSample.InvokeSafe(QueryStrategy);
        tabControl1.SelectTab(0);
      } catch (Exception e) {
        outputTextBox.Text = e.Message;
        tabControl1.SelectTab(1);
      }
      finally {
        OutputTreeView = null;
        this.Cursor = hold;
      }

    }


    #region Added Code

    #region Menu Handlers

    private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
      this.Close();
    }

    private void devForceTraceViewerToolStripMenuItem_Click(object sender, EventArgs e) {
      LaunchTraceViewer();
    }

    private void stuffCacheToolStripMenuItem_Click(object sender, EventArgs e) {
      LoadCache();
    }

    private void clearCacheToolStripMenuItem_Click(object sender, EventArgs e) {
      ClearCache();
    }

    private void saveCacheToDiskToolStripMenuItem_Click(object sender, EventArgs e) {
      SaveEntitySet();
    }

    private void restoreCacheFromDiskToolStripMenuItem_Click(object sender, EventArgs e) {
      RestoreEntitySet();
    }

    private void qsMenuItem_Click(object sender, EventArgs e) {
      var menuItem = sender as ToolStripMenuItem;
      var qs = menuItem.Tag as QueryStrategy;
      QueryStrategy = qs;
    }

    #endregion

    #region Cache Operations

    private void LoadCache() {
      this._toolStripStatusLabel.Text = "Loading cache...";
      Application.DoEvents();
      var origCursor = this.Cursor;
      try {
        this.Cursor = Cursors.WaitCursor;
        foreach (var harness in _harnesses.OfType<DevForceSampleHarness>()) {
          harness.LoadCache();
        }
      } finally {
        this.Cursor = origCursor;
      }

      // Note that there are other types (e.g., Employees) referenced in some of the queries
      // that we're not loading. Because of that, those queries won't work disconnected.
      UpdateMenus();
    }

    private void ClearCache() {
      _entityManager.Clear();
    }


    private void SaveEntitySet() {
      try {
        _entityManager.CacheStateManager.SaveCacheState(_entitySetPathName);
        MessageBox.Show("Contents of cache save to local disk file " + _entitySetPathName);
      }
      catch (Exception) {
        MessageBox.Show("Unable to save contents of cache to local disk file " + _entitySetPathName);
      }
    }

    private void RestoreEntitySet() {
      
      try {
        _entityManager.CacheStateManager.RestoreCacheState(_entitySetPathName, RestoreStrategy.Normal);
        MessageBox.Show("Contents of cache restored from local disk file " + _entitySetPathName);
      }
      catch (Exception) {
        MessageBox.Show("Unable to restore data from local disk file " + _entitySetPathName);
      }
    }


    #endregion

    #region UI state maintenance and utility

    private void InitImageList() {
      this.imageListConnection.Images.Add("Connected", SampleQueries.Resources01.connected);
      this.imageListConnection.Images.Add("Disconnected", SampleQueries.Resources01.disconnected);
    }

    private void UpdateMenus() {
      
      UpdateQsMenu(QueryStrategy);
      
      // Always available items:
      this.clearCacheToolStripMenuItem.Enabled = true;
      this.saveCacheToDiskToolStripMenuItem.Enabled = true;
      this.restoreCacheFromDiskToolStripMenuItem.Enabled = true;

    }

    private void UpdateQsMenu(QueryStrategy queryStrategy) {
      foreach (var item in this.qsCacheOnlyMenuItem.Owner.Items) {
        var mi = item as ToolStripMenuItem;
        mi.Checked = mi.Tag.Equals(queryStrategy);
        if (mi.Checked) {
          _toolStripStatusLabel.Text = mi.Text;
        }
      }
      bool connected = QueryStrategy != QueryStrategy.CacheOnly;
      this.stuffCacheToolStripMenuItem.Enabled = connected;
      if (connected) {
        this._toolStripStatusLabel.Image = imageListConnection.Images["Connected"];
      } else {
        this._toolStripStatusLabel.Image = imageListConnection.Images["Disconnected"];
      }
      // insure that treeView is never showing data retrieved for an earlier query strategy.
      outputTextBox.Clear();
      treeViewOutput.Nodes.Clear();
    }


    private void ConfigureStatusStrip() {
      this.statusStrip1.Items.AddRange(new System.Windows.Forms.ToolStripItem[] {
            _toolStripStatusLabel});
    }
    
    private static void LaunchTraceViewer() {
      IdeaBlade.DevTools.TraceViewer.TraceViewerForm tv =
        new IdeaBlade.DevTools.TraceViewer.TraceViewerForm();
      tv.Show();
    }

    #endregion

    #region Private Fields
    List<SampleHarness> _harnesses;
    EntityManager _entityManager;
    Color _colorConnectedNo = System.Drawing.Color.Red;
    Color _colorConnectedYes = System.Drawing.Color.Green;
    string _entitySetPathName = "C:\\EntitySetCache.bin";
    ToolStripStatusLabel _toolStripStatusLabel = new ToolStripStatusLabel();
    #endregion

  
    #endregion


  }

  abstract class DevForceSampleHarness : SampleHarness {

    public override void InitSample(Object initializationData) {
      var queryStrategy = (QueryStrategy)initializationData;
      EntityManager.DefaultQueryStrategy = queryStrategy;
      if (queryStrategy == QueryStrategy.CacheOnly) {
        EntityManager.Disconnect();
      } else {
        EntityManager.Connect();
      }
    }

    public abstract EntityManager EntityManager { get; }

    public virtual void LoadCache() {
    }

  }

}