'Copyright (C) Microsoft Corporation.  All rights reserved.

Imports System.Collections
Imports System.Reflection
Imports System.Text
Imports System.IO
Imports IdeaBlade.EntityModel

Namespace SampleSupport



  Public Class SampleHarness
	  Implements IEnumerable(Of Sample)
	Private ReadOnly samples As IDictionary(Of Integer, Sample) = New Dictionary(Of Integer, Sample)()

	Private ReadOnly _Title As String

	Public Sub New()
	  Dim samplesType As Type = Me.GetType()

	  _Title = "Samples"
	  Dim prefix As String = "Sample"
      Dim codeFile As String = samplesType.Name & ".vb"

	  For Each a As Attribute In samplesType.GetCustomAttributes(False)
		If TypeOf a Is TitleAttribute Then
		  _Title = (CType(a, TitleAttribute)).Title
		ElseIf TypeOf a Is PrefixAttribute Then
		  prefix = (CType(a, PrefixAttribute)).Prefix
		End If
	  Next a

	  Dim allCode As String = readFile(Application.StartupPath & "\..\..\" & codeFile)
	  Dim m As Integer = 1

	  Dim methods() As MethodInfo = samplesType.GetMethods(BindingFlags.Public Or BindingFlags.Instance Or BindingFlags.DeclaredOnly Or BindingFlags.Static)
	  For Each method As MethodInfo In methods
		If method.Name.StartsWith(prefix, StringComparison.OrdinalIgnoreCase) Then

		  Dim methodCategory As String = "Miscellaneous"
		  Dim methodTitle As String = prefix & " Sample " & m
		  Dim methodDescription As String = "See code."
		  Dim linkedMethods As New List(Of MethodInfo)()
		  Dim linkedClasses As New List(Of Type)()

		  For Each a As Attribute In method.GetCustomAttributes(False)
			If TypeOf a Is CategoryAttribute Then
			  methodCategory = (CType(a, CategoryAttribute)).Category
			ElseIf TypeOf a Is TitleAttribute Then
			  methodTitle = (CType(a, TitleAttribute)).Title
			ElseIf TypeOf a Is DescriptionAttribute Then
			  methodDescription = (CType(a, DescriptionAttribute)).Description
			ElseIf TypeOf a Is LinkedMethodAttribute Then
			  Dim linked As MethodInfo = samplesType.GetMethod((CType(a, LinkedMethodAttribute)).MethodName, (BindingFlags.Public Or BindingFlags.NonPublic) Or (BindingFlags.Static Or BindingFlags.Instance))
			  If linked IsNot Nothing Then
				linkedMethods.Add(linked)
			  End If
			ElseIf TypeOf a Is LinkedClassAttribute Then
			  Dim linked As Type = samplesType.GetNestedType((CType(a, LinkedClassAttribute)).ClassName)
			  If linked IsNot Nothing Then
				linkedClasses.Add(linked)
			  End If
			End If
		  Next a

		  Dim methodCode As New StringBuilder()
          methodCode.Append(getCodeBlock(allCode, "Sub " & method.Name))

		  For Each lm As MethodInfo In linkedMethods
			methodCode.Append(Environment.NewLine)
			methodCode.Append(getCodeBlock(allCode, shortTypeName(lm.ReturnType.FullName) & " " & lm.Name))
		  Next lm

		  For Each lt As Type In linkedClasses
			methodCode.Append(Environment.NewLine)
			methodCode.Append(getCodeBlock(allCode, "class " & lt.Name))
		  Next lt

		  Dim sample As New Sample(Me, method, methodCategory, methodTitle, methodDescription, methodCode.ToString())

		  samples.Add(m, sample)
		  m += 1
		End If
	  Next method
	End Sub

	Private Shared Function readFile(ByVal path As String) As String
	  Dim fileContents As String
	  If File.Exists(path) Then
		Using reader As StreamReader = File.OpenText(path)
		  fileContents = reader.ReadToEnd()
		End Using
	  Else
		fileContents = ""
	  End If

	  Return fileContents
	End Function

	Private Shared Function shortTypeName(ByVal typeName As String) As String
	  Dim isAssemblyQualified As Boolean = typeName.Chars(0) = "["c
	  If isAssemblyQualified Then
		Dim commaPos As Integer = typeName.IndexOf(","c)
		Return shortTypeName(typeName.Substring(1, commaPos - 1))
	  Else
		Dim isGeneric As Boolean = typeName.Contains("`")
		If isGeneric Then
		  Dim backTickPos As Integer = typeName.IndexOf("`"c)
		  Dim leftBracketPos As Integer = typeName.IndexOf("["c)
		  Dim typeParam As String = shortTypeName(typeName.Substring(leftBracketPos + 1, typeName.Length - leftBracketPos - 2))
		  Return shortTypeName(typeName.Substring(0, backTickPos)) & "<" & typeParam & ">"
		Else
		  Select Case typeName
			Case "System.Void"
				Return "void"
			Case "System.Int16"
				Return "short"
			Case "System.Int32"
				Return "int"
			Case "System.Int64"
				Return "long"
			Case "System.Single"
				Return "float"
			Case "System.Double"
				Return "double"
			Case "System.String"
				Return "string"
			Case "System.Char"
				Return "char"
			Case "System.Boolean"
				Return "bool"

			' other primitive types omitted 

			Case Else
			  Dim lastDotPos As Integer = typeName.LastIndexOf("."c)
			  Dim lastPlusPos As Integer = typeName.LastIndexOf("+"c)
			  Dim startPos As Integer = Math.Max(lastDotPos, lastPlusPos) + 1
			  Return typeName.Substring(startPos, typeName.Length - startPos)
		  End Select
		End If
	  End If
	End Function

	Private Shared Function getCodeBlock(ByVal allCode As String, ByVal blockName As String) As String
	  Dim blockStart As Integer = allCode.IndexOf(blockName, StringComparison.OrdinalIgnoreCase)

	  If blockStart = -1 Then
        Return "'' " & blockName & " code not found"
	  End If
	  blockStart = allCode.LastIndexOf(Environment.NewLine, blockStart, StringComparison.OrdinalIgnoreCase)
	  If blockStart = -1 Then
		blockStart = 0
	  Else
		blockStart += Environment.NewLine.Length
	  End If

	  Dim pos As Integer = blockStart
      Dim bFoundEndSub As Boolean = False
      Dim c As Char
      Dim d As String
      Dim e As String
	  Do
		pos += 1

        c = allCode.Chars(pos)
        d = allCode.Substring(pos, 3)
        e = allCode.Substring(pos, 7)
        Select Case e
          Case "End Sub"
            bFoundEndSub = True
        End Select
      Loop While pos < allCode.Length AndAlso Not bFoundEndSub

      Dim blockEnd As Integer = pos + 7

	  Dim blockCode As String = allCode.Substring(blockStart, blockEnd - blockStart + 1)

	  Return removeIndent(blockCode)
	End Function

	Private Shared Function removeIndent(ByVal code As String) As String
	  Dim indentSpaces As Integer = 0
	  Do While code.Chars(indentSpaces) = " "c
		indentSpaces += 1
	  Loop

	  Dim builder As New StringBuilder()
	  Dim codeLines() As String = code.Split(New String() { Environment.NewLine }, StringSplitOptions.None)
	  For Each line As String In codeLines
		If indentSpaces < line.Length Then
		  builder.AppendLine(line.Substring(indentSpaces))
		Else
		  builder.AppendLine()
		End If
	  Next line

	  Return builder.ToString()
	End Function


	Public Overridable Sub InitSample(ByVal initializationData As Object)
	End Sub

	Public Overridable Sub HandleException(ByVal e As Exception)
	  Console.Write(e)
	End Sub


	Public ReadOnly Property Title() As String
	  Get
		  Return _Title
	  End Get
	End Property




	Default Public ReadOnly Property Item(ByVal index As Integer) As Sample
	  Get
		  Return samples(index)
	  End Get
	End Property

    Private Function GetEnumerator1() As IEnumerator(Of Sample) Implements IEnumerable(Of SampleSupport.Sample).GetEnumerator
      Return samples.Values.GetEnumerator()
    End Function


    Private Function GetEnumerator() As IEnumerator Implements IEnumerable.GetEnumerator
      Return samples.Values.GetEnumerator()
    End Function
  End Class

  Public Class Sample
	Private ReadOnly _Harness As SampleHarness
	Private ReadOnly _Method As MethodInfo
	Private ReadOnly _Category As String
	Private ReadOnly _Title As String
	Private ReadOnly _Description As String
	Private ReadOnly _Code As String


	Public Sub New(ByVal harness As SampleHarness, ByVal method As MethodInfo, ByVal category As String, ByVal title As String, ByVal description As String, ByVal code As String)
	  Me._Harness = harness
	  Me._Method = method
	  Me._Category = category
	  Me._Title = title
	  Me._Description = description
	  Me._Code = code
	End Sub


	Public ReadOnly Property Harness() As SampleHarness
	  Get
		  Return _Harness
	  End Get
	End Property

	Public ReadOnly Property Method() As MethodInfo
	  Get
		  Return _Method
	  End Get
	End Property

	Public ReadOnly Property Category() As String
	  Get
		  Return _Category
	  End Get
	End Property

	Public ReadOnly Property Title() As String
	  Get
		  Return _Title
	  End Get
	End Property

	Public ReadOnly Property Description() As String
	  Get
		  Return _Description
	  End Get
	End Property

	Public ReadOnly Property Code() As String
	  Get
		  Return _Code
	  End Get
	End Property

	Public Sub InvokeSafe(ByVal initializationData As Object)
	  _Harness.InitSample(initializationData)
	  Try
		Invoke()
	  Catch e As TargetInvocationException
		Throw e.InnerException
	  Catch
		Throw
	  End Try
	End Sub

	Private Sub Invoke()
	  _Method.Invoke(Me._Harness, Nothing)
	End Sub

	Public Overrides Function ToString() As String
	  Return Title
	End Function
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method Or AttributeTargets.Class, AllowMultiple := False)>
  Public NotInheritable Class TitleAttribute
	  Inherits Attribute
	Public Sub New(ByVal title As String)
	  Me.Title = title
	End Sub

	Public Property Title() As String
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method Or AttributeTargets.Class, AllowMultiple := False)>
  Public NotInheritable Class PrefixAttribute
	  Inherits Attribute
	Public Sub New(ByVal prefix As String)
	  Me.Prefix = prefix
	End Sub

	Public Property Prefix() As String
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method, AllowMultiple := False)>
  Public NotInheritable Class CategoryAttribute
	  Inherits Attribute
	Public Sub New(ByVal category As String)
	  Me.Category = category
	End Sub

	Public Property Category() As String
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method, AllowMultiple := False)>
  Public NotInheritable Class DescriptionAttribute
	  Inherits Attribute
	Public Sub New(ByVal description As String)
	  Me.Description = description
	End Sub

	Public Property Description() As String
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method, AllowMultiple := True)>
  Public NotInheritable Class LinkedMethodAttribute
	  Inherits Attribute

	Public Sub New(ByVal methodName As String)
	  Me.MethodName = methodName
	End Sub

	Public Property MethodName() As String
  End Class

  <Global.System.AttributeUsage(AttributeTargets.Method, AllowMultiple := True)>
  Public NotInheritable Class LinkedClassAttribute
	  Inherits Attribute
	Public Sub New(ByVal className As String)
	  Me.ClassName = className
	End Sub

	Public Property ClassName() As String
  End Class
End Namespace
